# frozen_string_literal: true

require "webrick"
require "sinatra"
require "docdsl"
require "webrick/https"
require "openssl"

# Virtual server hosting virtual service defined through files
class VirtualServer < Sinatra::Base
  set :server, "webrick"
  set :bind, "0.0.0.0"

  register Sinatra::DocDsl

  if GlobalSettings.https
    def self.own_certs(webrick_options)
      puts "Using cert from #{GlobalSettings.cert}"
      cert = OpenSSL::X509::Certificate.new File.read GlobalSettings.cert
      pkey = OpenSSL::PKey::RSA.new File.read GlobalSettings.key
      webrick_options[:SSLCertificate] = cert
      webrick_options[:SSLPrivateKey] = pkey
      webrick_options
    end

    def self.determine_certs(webrick_options)
      if GlobalSettings.key && GlobalSettings.cert
        webrick_options = own_certs webrick_options
      else
        puts "Using self signed cert"
        webrick_options[:ServerName] = "localhost"
        webrick_options[:SSLCertName] = "/CN=localhost"
      end
      webrick_options
    end

    # Run as https with self signed cert
    def self.run!
      logger = WEBrick::Log.new(nil, WEBrick::BasicLog::WARN)
      webrick_options = { Port: port, SSLEnable: true, Logger: logger }
      webrick_options = determine_certs webrick_options
      # TODO: Following run does not work on Ruby 3
      Rack::Handler::WEBrick.run(self, webrick_options) do |server|
        %i[INT TERM].each { |sig| trap(sig) { server.stop } }
        server.threaded = settings.threaded if server.respond_to? :threaded=
        set :running, true
      end
    end
  end

  page do
    title "File SV"
    header "Service virtualization created from #{Dir.pwd}"
    introduction 'Created using file_sv. See <a href="https://gitlab.com/samuel-garratt/file_sv">File SV</a>
for more details'
  end

  get "/favicon.ico" do
    send_file File.join(__dir__, "file_sv.ico")
  end

  doc_endpoint "/docs"

  # Output for endpoint, either a file or text content
  # @param [PlannedEndpoint] endpoint Planned endpoint to serve
  def output_for(endpoint, binding)
    endpoint.file? ? send_file(endpoint.serving_file_name) : endpoint.content(binding)
  end

  # Log endpoint. Return content and status code defined by endpoint
  # @param [PlannedEndpoint] endpoint Planned endpoint to serve
  def serve(endpoint, id = nil)
    message = "Using endpoint based on file #{endpoint.serving_file_name}."
    @id = id
    message += " Using param '#{@id}'" if id
    puts message
    [endpoint.status_code, output_for(endpoint, binding)]
  end

  SvPlan.endpoints.each do |_endpoint_path, endpoint_value|
    endpoint_value.each do |_method_name, endpoints|
      endpoint_base = endpoints[0]
      documentation "Endpoint #{endpoint_base.path}" do
        response "#{endpoints.size} kinds of response"
      end
      if endpoint_base.path.include? "#{File::Separator}:"
        send(endpoint_base.method, endpoint_base.path) do |id|
          endpoint = endpoints.sample
          serve endpoint, id
        end
      else
        send(endpoint_base.method, endpoint_base.path) do
          endpoint = endpoints.sample
          serve endpoint
        end
      end
    end
  end
end
